import {IViewLogicOptions, IComponentOptions, IViewLogicApplyOptions, IMethods} from "../../interface/IOptions";
import deepmerge from "deepmerge";
import {Observable, Subject} from "rxjs";
import { v4 as uuid } from 'uuid';
import RequestModel from "../../model/RequestModel";
import {LogicType, RequestSequenceType, RequestStatus} from "../../variable/enum";
import {compOptFromLogicOpt, getMethodName, getVarableNameFromRequest, methOptFromLogicOpt} from "../../tool/MethodTool";
import {BaseComponent, BaseLogic, createMethodsSet} from "../base";

const EMPTY_LOGIC_OPTION: IViewLogicOptions = {
    type: LogicType.MIXIN,
    requestListInspect: _ => _,
    requestSequence: {
        type: RequestSequenceType.CONCURRENCY,
        requestList: [
        ],
    },
};

const updateRequestStatus = (context: Vue, req: RequestModel, status: RequestStatus) => {
    const index = context.$data['_requestStatus'].findIndex(_ => {
        return _.req.methodName === req.methodName
    });
    context.$set(context.$data['_requestStatus'], index, {
        ...context.$data['_requestStatus'][index],
        status
    });
    context.$data[req.statusVarable] = status;
};

const BaseViewMethods: any & IMethods<BaseViewComponent> = {
    __initData(context){
        this.__fetchRequestList(context);
    },
    __fetchRequest(context, requestKey, options: any = {}){
        const {subject} = this.component;
        const requestList = this.options.requestList;
        const req = requestList.find(_ => _.key === requestKey);
        if (!req){
            console.warn(`没有找到 Request Key: ${requestKey}`)
            return;
        }
        const reqObservable = new Observable(subscriber => {
            const methodName = getMethodName(req);
            const id = uuid();
            const nextObj = {
                type: 'request',
                id,
                methodName,
            };
            subscriber.next(nextObj);
            subject.next(nextObj);
            if (!context[methodName]){
                throw new Error(`没有找到方法 ${methodName}`)
            }
            updateRequestStatus(context, req, RequestStatus.HOLD);
            context[methodName]().then(data => {
                const nextObj = {
                    type: 'response',
                    id,
                    methodName,
                    data,
                };
                this.__assignResponseData(context, req, data, options, function defaultAssign(context, req, data, options){
                    const {isAppend = false} = options;
                    const varable = req.varable;
                    if (isAppend){
                        // TODO: 数组改成 可迭代对象
                        if (!Array.isArray(context[varable])){
                            throw new Error('非Array对象不可进行追加数据')
                        }
                        context[varable] = context[varable].concat(data);
                    } else {
                        context[varable] = data;
                    }
                });
                updateRequestStatus(context, req, RequestStatus.FINISH);
                subscriber.next(nextObj);
                subject.next(nextObj);
            }, error => {
                subscriber.next({
                    type: 'response',
                    id,
                    methodName,
                    error,
                });
            });
        });
        reqObservable.subscribe((v) => {
            if (context.networkListener){
                context.networkListener.call(context, v)
            }
        });
    },
    __fetchRequestList(context, keyList?){
        let {requestList, requestType} = this.options;
        if (requestType === RequestSequenceType.CONCURRENCY){
            for (let index = 0; index < requestList.length; index++){
                const req: RequestModel = requestList[index];
                if (Array.isArray(keyList) && keyList.indexOf(req.key) === -1) continue;
                this.__fetchRequest(context, req.key, {})
            }
        }
    },
    __assignResponseData(context, req, data, options, defaultAssign){
        defaultAssign(context, req, data, options)
    }
};

class BaseViewComponent extends BaseComponent{
    protected options: IComponentOptions = null;
    protected logicOptions: IViewLogicApplyOptions;
    public subject: Subject<any> = null;
    constructor(options: IComponentOptions, logicOptions: IViewLogicApplyOptions) {
        super(options, logicOptions);
        this.options = options;
        this.logicOptions = logicOptions;
    }

    public created(context){
        this.subject = new Subject();
    }

    public mounted(context) {

    }

    public computed(context) {
        const {computedVariableObj} = this.options;
        return {
            ...computedVariableObj
        }
    }

    public data(context): Object{
        const {variablesObj} = this.options;
        return {
            ...variablesObj,
        };
    }
}

class BaseViewLogic extends BaseLogic<IViewLogicOptions>{
    protected static defaultOptions: IViewLogicOptions = null;
    protected static Component = null;
    protected static Methods = null;

    constructor(options) {
        super(options);
    }

    public getOptions(): IViewLogicApplyOptions{
        let {options} = this;
        const {requestSequence, requestListInspect} = options;
        options.requestSequence = {
            ...requestSequence,
            requestList: requestListInspect(requestSequence.requestList)
        };
        return options
    }

    protected logAllProperty(){
        console.log('子类未实现 打印所有可用属性 方法')
    }

    protected createLogic(){
        const {Component, Methods} = Object.getPrototypeOf(this).constructor;
        if (!Component || !Methods){
            throw new Error('子类需要提供 Component 及 Methods 定义');
        }
        const options = this.getOptions();
        const component = new Component(compOptFromLogicOpt(options), options);
        const methods = createMethodsSet(Methods, methOptFromLogicOpt(options), component, BaseViewMethods);
        // 退出的对象上下文改变成 vue 上下文
        return {
            created() {
                component.created(this);
            },
            mounted(){
                component.mounted(this);
            },
            data(){
                return component.data(this)
            },
            computed:{
                ...component.computed(this)
            },
            methods: {
                ...methods
            }
        }
    }

    static makeLogic<T>(opt?: IViewLogicOptions | ((IViewLogicOptions) => IViewLogicOptions)){
        const Class = Object.getPrototypeOf(this).constructor;
        let logic = typeof opt === 'function' ? new this(opt(Class.defaultOptions))
            : new this(opt || Class.defaultOptions || EMPTY_LOGIC_OPTION);
        return {
            main: logic.createLogic()
        };
    }
}

export {
    BaseViewLogic,
    BaseViewComponent,
    updateRequestStatus,
}
