import {IBaseLogicOptions, IComponentOptions, IMethods} from "@/interface/IOptions";
import {Subject} from "rxjs";
import deepmerge from "deepmerge";
import {LogicType} from "@/variable/enum";

const BaseMethods: any & IMethods<BaseComponent> = {
};

class BaseComponent<T = IBaseLogicOptions>{
    protected options: IComponentOptions = null;
    protected logicOptions: T;
    public subject: Subject<any> = null;
    constructor(options: IComponentOptions, logicOptions: T) {
        this.options = options;
        this.logicOptions = logicOptions;
    }

    public created(context){
        this.subject = new Subject();
    }

    public mounted(context) {
    }

    public computed(context) {
        const {computedVariableObj} = this.options;
        return {
            ...computedVariableObj
        }
    }

    public data(context): Object{
        const {variablesObj} = this.options;
        return {
            ...variablesObj,
        };
    }
}


class BaseLogic<T = IBaseLogicOptions>{
    protected options: T = null;
    protected static defaultOptions: IBaseLogicOptions = {type: LogicType.MIXIN};
    protected static Component = null;
    protected static Methods = null;

    constructor(options = {}) {
        this.options = deepmerge(Object.getPrototypeOf(this).constructor.defaultOptions, options);
    }

    public getOptions(): T{
        return this.options;
    }

    protected logAllProperty(){
        console.log('子类未实现 打印所有可用属性 方法')
    }

    protected createLogic(){
        const {Component, Methods} = Object.getPrototypeOf(this).constructor as {
            Component: {new(...args: any[]): BaseComponent},
            Methods: Object
        };
        if (!Component || !Methods){
            throw new Error('子类需要提供 Component 及 Methods 定义');
        }
        const options = this.getOptions();
        const component = new Component({
            logicOptions: options,
            variablesObj: {},
            computedVariableObj: {},
        }, options);
        const methods = createMethodsSet(Methods, {}, component);
        // 退出的对象上下文改变成 vue 上下文
        return {
            created() {
                component.created(this);
            },
            mounted(){
                component.mounted(this);
            },
            data(){
                return component.data(this)
            },
            computed:{
                ...component.computed(this)
            },
            methods: {
                ...methods
            }
        }
    }

    static makeLogic<T>(opt?: IBaseLogicOptions){
        const Class = Object.getPrototypeOf(this).constructor;
        let logic = new this(opt || Class.defaultOptions);
        return {
            main: logic.createLogic()
        };
    }
}

const createMethodsSet = (m, opt, comp, baseMethods = BaseMethods) => {
    const _m = Object.create(baseMethods, {
        options: {
            writable: false,
            configurable: true,
            value: opt
        },
        component: {
            writable: false,
            configurable: true,
            value: comp
        }
    });
    for (let key in m){
        if (m.hasOwnProperty(key)){
            _m[key] = m[key];
        }
    }
    const methodObj = {};
    for (let key in _m){
        if (_m.hasOwnProperty(key)){
            methodObj[key] = function(...args) {
                return _m[key].call(_m, this, ...args);
            };
        }
    }
    return methodObj
};

export {
    BaseLogic,
    BaseComponent,
    createMethodsSet,
}
