'use strict';
import LocatorModel from "@/model/LocatorModel";
import {LocatorType} from "@/variable/enum";
import {getMapConfig} from '@/util/ConfigManager';

type Coordinate<T = null | number> = {
  longitude?: T;
  latitude?: T;
}

const CoordinateNull: Coordinate<null> = {
  longitude: null,
  latitude: null,
};


interface LocationManagerOption{
  mapFrameworks?: any[]
  mapPlatforms?: LocatorType[]
  nativeLocationCallback?: (callback: (coor: Coordinate) => void) => void
  nativeLocation?: () => void
  nativeLocationIsNull?: (coor: Coordinate) => boolean
}

let instance: LocationManager = null;
class LocationManager {
  private nativeLocationCallback: (location: Coordinate) => void;
  private options: LocationManagerOption = null;

  private constructor (options: LocationManagerOption) {
    if (!instance) {
      instance = this;
      this.options = options = {
        ...getMapConfig(),
        ...options,
      };
      options.nativeLocationCallback((location) => {
        this.locationCallback(location)
      });
      this.nativeLocationCallback = null;
    }
    return instance
  }

  static prepareManager(options: LocationManagerOption = {}){
    if (instance){
      throw new Error('只能执行 prepareManager 一次');
    }
    return new LocationManager(options);
  }

  static defaultManager () {
    if (!instance){
      throw new Error('需要执行 prepareManager')
    }
    return instance;
  }

  locationCallback (location: Coordinate): void {
    if (this.nativeLocationCallback) {
      this.nativeLocationCallback(location)
    }
  }

  /**
   * app定位超时处理
   */
  async appLocationTimeOutPromise (timeout = 1500): Promise<Coordinate> {
    return new Promise((resolve) => {
      setTimeout(() => {
        resolve({longitude: null, latitude: null})
      }, timeout)
    })
  }

  getNativeLocation (): Promise<Coordinate> {
    const {nativeLocation} = this.options;
    return new Promise((resolve, reject) => {
      this.nativeLocationCallback = (location) => {
        resolve(location)
      };
      try {
        nativeLocation()
      } catch (e) {
        resolve(null);
      }
    })
  }

  nativeLocationNull (coordinate) {
    const {nativeLocationIsNull} = this.options;
    if (nativeLocationIsNull){
      return nativeLocationIsNull(coordinate);
    }
    return !!coordinate;
  }

  /**
   * 获取定位（经纬度）
   * @returns {Promise}
   */
  async getLocation (locatorList: LocatorModel[], options: any) {
    let openNativeLocation = false, {timeout} = options;
    for (let i = 0, l = locatorList.length; i < l; i++){
      const locator = locatorList[i];
      if (locator.key === LocatorType.NATIVE){
        openNativeLocation = true;
        timeout = locator.timeout ?? timeout;
      }
    }

    let coordinate: Coordinate = CoordinateNull;
    if (openNativeLocation) {
      coordinate = await Promise.race([this.appLocationTimeOutPromise(timeout), this.getNativeLocation()]).then((coordinate: Coordinate) => {
        let longitude = null;
        let latitude = null;
        if (!this.nativeLocationNull(coordinate)) {
          longitude = coordinate.longitude;
          latitude = coordinate.latitude
        }
        return {
          longitude,
          latitude,
        }
      }).catch(error => {
        console.log(error);
        return CoordinateNull;
      })
    }

    return new Promise((resolve, reject) => {
      const {longitude, latitude} = coordinate;
      this.handleLocationPromise(locatorList.filter(({key}) => key !== LocatorType.NATIVE), longitude, latitude).then(location => {
        // this.updateLocationStore(location)
        resolve(location)
      }, error => {
        // this.updateLocationStore()
        reject(error)
      })
    })
  }


  /**
   * web定位处理回调
   * @param longitude 定位到得经纬度
   * @param latitude 定位到的经纬度
   */
  handleLocationPromise (locatorList: LocatorModel[], longitude?, latitude?): Promise<Coordinate> {
    if (longitude && latitude) {
      return new Promise((resolve, reject) => {
        console.log('good, the native app location is enable');
        resolve({longitude, latitude})
      })
    } else {
      const {mapPlatforms, mapFrameworks} = this.options;
      return new Promise((resolve, reject) => {
        console.log('oops, the native app location is fly');

        // locatorList.map(({key}) => {
        //   const index = mapPlatforms.findIndex(*)
        // })

        // Promise.any([this.baiduLocatePromise()]).then(data => {
        //   console.log(data);
        // });
        // this.baiduLocatePromise().then(location => {
        //   resolve(location)
        // }, error => {
        //   reject(error)
        // })
      })
    }
  }

  /**
   * 浏览器定位
   */
  browserLocate () {
    return new Promise((resolve, reject) => {
      // navigator.geolocation.getCurrentPosition(location => {
      //   this.doSuccessCallback(location)
      // }, () => {
      //   this.doErrorCallback()
      // })
    })

  }

  /**
   * 百度定位
   */
  baiduLocatePromise (BMap) {
    return new Promise((resolve, reject) => {
      let geolocation = new BMap.Geolocation()
      geolocation.getCurrentPosition((options) => {
            if (geolocation.getStatus() === 0 && options.accuracy !== null) {
              resolve(options)
            } else {
              reject()
            }
          },
          () => {
            reject()
          },
          {timeout: 2000, maximumAge: 0})
    })
  }

  // updateLocationStore (location) {
  //   if (location) {
  //       this._hasLocation = true
  //     GlobalConstant.store.commit('UPDATE_LOCATION_MUTATION', {
  //       latitude: location.latitude,
  //       longitude: location.longitude,
  //       hasLocation: true
  //     })
  //   } else {
  //     this._hasLocation = false
  //     GlobalConstant.store.commit('UPDATE_LOCATION_MUTATION')
  //   }
  // }

  /**
   * 获取当前城市
   * @param callback
   */
  // getLocationCity (callback) {
  //   NetworkCity.smartCityByCoordinate().then(data => {
  //     if (callback) {
  //       callback(null, data)
  //     }
  //   }, error => {
  //     if (callback) {
  //       callback(error, null)
  //     }
  //   })
  // }

  /**
   * 底部弹出窗口
   * @param text 弹出窗口的内容
   */
  // bottomAlert (text) {
  //   if (UrlTool.getQueryString('isTest') === 'true') {
  //     Toast({
  //       message: text,
  //       position: 'bottom',
  //       duration: 2000
  //     })
  //   }
  // }

}

export default LocationManager
